<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 449 $
 * $Id: ClientPaymentFactory.class.php 449 2006-12-19 19:07:54Z ipso $
 * $Date: 2006-12-19 11:07:54 -0800 (Tue, 19 Dec 2006) $
 */
class ClientPaymentFactory extends Factory {
	protected $table = 'client_payment';
	protected $pk_sequence_name = 'client_payment_id_seq'; //PK Sequence name

	protected $status_options = array(
										10 => 'Enabled',
										20 => 'Disabled - Failed',
										60 => 'Disabled'
									);

	protected $type_options = array(
										30 => 'Credit Card',
										//40 => 'Bank Transfer'
									);

	protected $credit_card_type_options = array(
										100 => 'Visa',
										101 => 'MasterCard',
										102 => 'American Express',
										103 => 'Discover',
									);

	protected $client_validator_msg = 'Client is invalid';
	protected $client_validator_label = 'client';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $creditcardtype_validator_options_msg = 'Incorrect Credit Card Type';
	protected $creditcardtype_validator_label = 'credit_card_type';

	protected $name_validator_regex = '/^[a-zA-Z -\.]{1,250}$/i';

	protected $creditcardname_validator_regex_msg = 'Name contains invalid characters';
	protected $creditcardname_validator_length_msg = 'Name is too short or too long';
	protected $creditcardname_validator_label = 'cc_name';

	protected $creditcardnumber_validator_isint_msg = 'Incorrect Credit Card Number';
	protected $creditcardnumber_validator_label = 'cc_number';

	protected $creditcardexpire_validator_isdate_msg = 'Incorrect Expiry Date';
	protected $creditcardexpire_validator_label = 'cc_expire';

	protected $creditcardcheck_validator_isint_msg = 'Incorrect Credit Card Check Digits';
	protected $creditcardcheck_validator_label = 'cc_check';

	protected $creditcardbankphone_validator_isint_msg = 'Incorrect Credit Card Bank Phone';
	protected $creditcardbankphone_validator_label = 'cc_bank_phone';

	protected $institution_validator_length_msg = 'Invalid institution number length';
	protected $institution_validator_numeric_msg = 'Invalid institution number, must be digits only';
	protected $institution_validator_label = 'institution';

	protected $transit_validator_length_msg = 'Invalid transit number length';
	protected $transit_validator_numeric_msg = 'Invalid transit number, must be digits only';
	protected $transit_validator_label = 'transit';

	protected $account_validator_length_msg = 'Invalid account number length';
	protected $account_validator_label = 'account';

	protected $default_validator_unique_msg = 'There is already a default payment method for this client';
	protected $default_validator_label = 'default';

	protected $note_validator_length_msg = 'Note is too short or too long';
	protected $note_validator_label = 'note';

	function getClient() {
		if ( isset($this->data['client_id']) ) {
			return $this->data['client_id'];
		}

		return FALSE;
	}
	function setClient($id) {
		$id = trim($id);

		Debug::Text('Client ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$clf = new ClientListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->client_validator_label,
													$clf->getByID($id),
													$this->client_validator_msg
													) ) {

			$this->data['client_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getStatus() {
		if ( isset($this->data['status_id']) ) {
			return $this->data['status_id'];
		}

		return FALSE;
	}
	function setStatus($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('status') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$value,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->data['status_id'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getType() {
		if ( isset($this->data['type_id']) ) {
			return $this->data['type_id'];
		}

		return FALSE;
	}
	function setType($type) {
		$type = trim($type);

		$key = Option::getByValue($type, $this->getOptions('type') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$type,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $type;

			return TRUE;
		}

		return FALSE;
	}

	function getCreditCardType() {
		if ( isset($this->data['cc_type_id']) ) {
			return $this->data['cc_type_id'];
		}

		return FALSE;
	}
	function setCreditCardType($type) {
		$type = trim($type);

		$key = Option::getByValue($type, $this->getOptions('credit_card_type') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->creditcardtype_validator_label,
											$type,
											$this->creditcardtype_validator_options_msg,
											$this->getOptions('credit_card_type')) ) {

			$this->data['cc_type_id'] = $type;

			return TRUE;
		}

		return FALSE;
	}

	function getCreditCardFirstName() {
		$split_name = explode( ' ', $this->getCreditCardName() );
		if ( isset($split_name[0]) ) {
			return $split_name[0];
		}

		return FALSE;
	}

	function getCreditCardLastName() {
		$split_name = explode( ' ', $this->getCreditCardName() );
		if ( isset($split_name[(count($split_name)-1)]) ) {
			return $split_name[(count($split_name)-1)];
		}

		return FALSE;
	}

	function getCreditCardName() {
		if ( isset($this->data['cc_name']) ) {
			return $this->data['cc_name'];
		}

		return FALSE;
	}
	function setCreditCardName($value) {
		$value = trim($value);

		if 	(	$this->Validator->isRegEx(		$this->creditcardname_validator_label,
												$value,
												$this->creditcardname_validator_regex_msg,
												$this->name_validator_regex)
				AND
					$this->Validator->isLength(		$this->creditcardname_validator_label,
													$value,
													$this->creditcardname_validator_length_msg,
													2,
													50) ) {

			$this->data['cc_name'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getSecureCreditCardNumber() {
		$total = strlen($this->getCreditCardNumber())-8;
		Debug::Text('Start: '. 4 .' Total Hidden: '. $total .' Length: '. strlen($this->getCreditCardNumber()), __FILE__, __LINE__, __METHOD__,10);
		return str_replace( substr( $this->getCreditCardNumber(), 4, $total ), str_repeat('*', $total), $this->getCreditCardNumber() );
	}
	function getCreditCardNumber() {
		if ( isset($this->data['cc_number']) ) {
			return Misc::decrypt( $this->data['cc_number'] );
		}

		return FALSE;
	}
	function setCreditCardNumber($value) {
		$value = trim($value);

		//If *'s are in the CC number, skip setting it
		//This allows them to change other data without seeing the CC number.
		if ( strpos( $value, '*') !== FALSE  ) {
			return FALSE;
		}
		
		//Pull out only digits and periods.
		$value = $this->Validator->stripNonFloat($value);

		if (	$this->Validator->isFloat(	$this->creditcardnumber_validator_label,
											$value,
											$this->creditcardnumber_validator_isint_msg)
				AND
				$this->Validator->isLength(	$this->creditcardnumber_validator_label,
											$value,
											$this->creditcardnumber_validator_isint_msg,
											12,
											20)

				) {

			$this->data['cc_number'] = Misc::encrypt( $value );

			return TRUE;
		}

		return FALSE;
	}


	function getCreditCardExpireMonth() {
		return date('m', $this->getCreditCardExpire() );
	}

	function getCreditCardExpireYear() {
		return date('Y', $this->getCreditCardExpire() );
	}

	function getCreditCardExpire() {
		if ( isset($this->data['cc_expire']) ) {
			return TTDate::strtotime($this->data['cc_expire']);
		}

		return FALSE;
	}
	function setCreditCardExpire($epoch) {
		$epoch = trim($epoch);

		if 	(	$this->Validator->isDate(		$this->creditcardexpire_validator_label,
												$epoch,
												$this->creditcardexpire_validator_isdate_msg) ) {

			$this->data['cc_expire'] = $epoch;

			return TRUE;
		}

		return FALSE;
	}

	function getCreditCardCheck() {
		if ( isset($this->data['cc_check']) ) {
			return $this->data['cc_check'];
		}

		return FALSE;
	}
	function setCreditCardCheck($value) {
		$value = trim($value);
		
		//Pull out only digits and periods.
		$value = $this->Validator->stripNonFloat($value);

		if (	$value == ''
				OR
				$this->Validator->isFloat(	$this->creditcardcheck_validator_label,
											$value,
											$this->creditcardcheck_validator_isint_msg) ) {

			$this->data['cc_check'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getCreditCardBankPhone() {
		if ( isset($this->data['cc_bank_phone']) ) {
			return $this->data['cc_bank_phone'];
		}

		return FALSE;
	}
	function setCreditCardBankPhone($value) {
		$value = trim($value);

		//Pull out only digits and periods.
		$value = $this->Validator->stripNonFloat($value);

		if (	$value == ''
				OR
				$this->Validator->isFloat(	$this->creditcardbankphone_validator_label,
											$value,
											$this->creditcardbankphone_validator_isint_msg) ) {

			$this->data['cc_bank_phone'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getInstitution() {
		if ( isset($this->data['bank_institution']) ) {
			return $this->data['bank_institution'];
		}

		return FALSE;
	}
	function setInstitution($value) {
		$value = trim($value);

		if (
						$this->Validator->isNumeric(	$this->institution_validator_label,
														$value,
														$this->institution_validator_numeric_msg)
				AND
						$this->Validator->isLength(		$this->institution_validator_label,
														$value,
														$this->institution_validator_length_msg,
														2,
														3)
			) {

			$this->data['bank_institution'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getTransit() {
		if ( isset($this->data['bank_transit']) ) {
			return $this->data['bank_transit'];
		}

		return FALSE;
	}
	function setTransit($value) {
		$value = trim($value);

		if (
						$this->Validator->isNumeric(	$this->transit_validator_label,
														$value,
														$this->transit_validator_numeric_msg)
				AND
						$this->Validator->isLength(		$this->transit_validator_label,
														$value,
														$this->transit_validator_length_msg,
														2,
														5)
			) {

			$this->data['bank_transit'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getSecureAccount() {
		$account = str_replace( substr($this->getAccount(),2,5), '***', $this->getAccount() );
		return $this->getTransit().' '.$this->getInstitution().' '.$account;
	}
	function getAccount() {
		if ( isset($this->data['bank_account']) ) {
			return $this->data['bank_account'];
		}

		return FALSE;
	}
	function setAccount($value) {
		$value = trim($value);

		if (
						$this->Validator->isLength(		$this->account_validator_label,
														$value,
														$this->account_validator_length_msg,
														5,
														12)
			) {

			$this->data['bank_account'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function isUniqueDefault() {
		$ph = array(
					'client_id' => $this->getClient(),
					);

		$query = 'select id from '. $this->getTable() .' where client_id = ? AND is_default = 1 AND deleted=0';
		$id = $this->db->GetOne($query, $ph);
		Debug::Arr($id,'Unique Client Payment Default: '. $id, __FILE__, __LINE__, __METHOD__,10);

		if ( $id === FALSE ) {
			return TRUE;
		} else {
			if ($id == $this->getId() ) {
				return TRUE;
			}
		}

		return FALSE;
	}

	function getDefault() {
		return $this->fromBool( $this->data['is_default'] );
	}
	function setDefault($bool) {

		if 	(
				$bool == TRUE
				AND
				$this->Validator->isTrue(		$this->default_validator_label,
												$this->isUniqueDefault(),
												$this->default_validator_unique_msg
												)
			) {

			$this->data['is_default'] = $this->toBool(TRUE);

			return TRUE;
		}

		$this->data['is_default'] = $this->toBool(FALSE);

		return TRUE;
	}

	function getNote() {
		if ( isset($this->data['note']) ) {
			return $this->data['note'];
		}
	}
	function setNote($val) {
		$val = trim($val);

		if 	(	$val == ''
				OR
				$this->Validator->isLength(		$this->note_validator_label,
												$val,
												$this->note_validator_length_msg,
												0,
												1024) ) {

			$this->data['note'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function postSave() {
		$this->removeCache( $this->getId() );

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Client Payment Information' , NULL, $this->getTable() );
	}

}
?>
