<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 114 $
 * $Id: PolicyGroupFactory.class.php 114 2006-09-15 23:47:37Z ipso $
 * $Date: 2006-09-15 16:47:37 -0700 (Fri, 15 Sep 2006) $
 */
class PolicyGroupFactory extends Factory {
	protected $table = 'policy_group';
	protected $pk_sequence_name = 'policy_group_id_seq'; //PK Sequence name

	protected $company_obj = NULL;

	protected $company_validator_msg = 'Company is invalid';
	protected $company_validator_label = 'company';

	protected $name_validator_unique_msg = 'Name is already in use';
	protected $name_validator_islength_msg = 'Name is invalid';
	protected $name_validator_label = 'name';

	protected $user_validator_msg = 'Selected Employee is invalid';
	protected $user_validator_label = 'user';

	protected $overtimepolicy_validator_msg = 'Selected Overtime Policy is invalid';
	protected $overtimepolicy_validator_label = 'over_time_policy';

	protected $premiumpolicy_validator_msg = 'Selected Premium Policy is invalid';
	protected $premiumpolicy_validator_label = 'premium_policy';

	protected $roundintervalpolicy_validator_msg = 'Selected Interval Rounding Policy is invalid';
	protected $roundintervalpolicy_validator_label = 'round_interval_policy';

	protected $mealpolicy_validator_msg = 'Meal Policy is invalid';
	protected $mealpolicy_validator_label = 'meal_policy';

	protected $exceptionpolicycontrol_validator_msg = 'Exception Policy is invalid';
	protected $exceptionpolicycontrol_validator_label = 'exception_policy';

	protected $holidaypolicy_validator_msg = 'Holiday Policy is invalid';
	protected $holidaypolicy_validator_label = 'holiday_policy';

	function getCompanyObject() {
		if ( is_object($this->company_obj) ) {
			return $this->company_obj;
		} else {
			$clf = new CompanyListFactory();
			$this->company_obj = $clf->getById( $this->getCompany() )->getCurrent();

			return $this->company_obj;
		}
	}

	function getCompany() {
		if ( isset($this->data['company_id']) ) {
			return $this->data['company_id'];
		}

		return FALSE;
	}
	function setCompany($id) {
		$id = trim($id);

		Debug::Text('Company ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$clf = new CompanyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->company_validator_label,
													$clf->getByID($id),
													$this->company_validator_msg
													) ) {

			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getName() {
		if ( isset($this->data['name']) ) {
			return $this->data['name'];
		}

		return FALSE;
	}
	function setName($name) {
		$name = trim($name);
		if (	$this->Validator->isLength(	$this->name_validator_label,
											$name,
											$this->name_validator_islength_msg,
											2,50)
						) {

			$this->data['name'] = $name;

			return TRUE;
		}

		return FALSE;
	}

	function getUser() {
		$pgulf = new PolicyGroupUserListFactory();
		$pgulf->getByPolicyGroupId( $this->getId() );
		foreach ($pgulf as $obj) {
			$list[] = $obj->getUser();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setUser($ids) {
		Debug::text('Setting User IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if ( is_array($ids) ) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$pgulf = new PolicyGroupUserListFactory();
				$pgulf->getByPolicyGroupId( $this->getId() );

				$tmp_ids = array();
				foreach ($pgulf as $obj) {
					$id = $obj->getUser();
					Debug::text('Policy ID: '. $obj->getPolicyGroup() .' ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			}

			//Insert new mappings.
			$ulf = new UserListFactory();

			foreach ($ids as $id) {
				if ( isset($ids) AND !in_array($id, $tmp_ids) ) {
					$pguf = new PolicyGroupUserFactory();
					$pguf->setPolicyGroup( $this->getId() );
					$pguf->setUser( $id );

					$obj = $ulf->getById( $id )->getCurrent();

					if ($this->Validator->isTrue(		$this->user_validator_label,
														$pguf->Validator->isValid(),
														$this->user_validator_msg.' ('. $obj->getFullName() .')' )) {
						$pguf->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No User IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function getOverTimePolicy() {
		$pgotplf = new PolicyGroupOverTimePolicyListFactory();
		$pgotplf->getByPolicyGroupId( $this->getId() );
		foreach ($pgotplf as $obj) {
			$list[] = $obj->getOverTimePolicy();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setOverTimePolicy($ids) {
		Debug::text('Setting OTP IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if ( is_array($ids) ) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$pgotplf = new PolicyGroupOverTimePolicyListFactory();
				$pgotplf->getByPolicyGroupId( $this->getId() );

				$tmp_ids = array();
				foreach ($pgotplf as $obj) {
					$id = $obj->getOverTimePolicy();
					Debug::text('Policy ID: '. $obj->getPolicyGroup() .' ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			}

			//Insert new mappings.
			$otplf = new OverTimePolicyListFactory();

			foreach ($ids as $id) {
				if ( isset($ids) AND !in_array($id, $tmp_ids) ) {
					$pgotpf = new PolicyGroupOverTimePolicyFactory();
					$pgotpf->setPolicyGroup( $this->getId() );
					$pgotpf->setOverTimePolicy( $id );

					$obj = $otplf->getById( $id )->getCurrent();

					if ($this->Validator->isTrue(		$this->overtimepolicy_validator_label,
														$pgotpf->Validator->isValid(),
														$this->overtimepolicy_validator_msg.' ('. $obj->getName() .')' )) {
						$pgotpf->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No OTP IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function getPremiumPolicy() {
		$pgpplf = new PolicyGroupPremiumPolicyListFactory();
		$pgpplf->getByPolicyGroupId( $this->getId() );
		foreach ($pgpplf as $obj) {
			$list[] = $obj->getPremiumPolicy();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setPremiumPolicy($ids) {
		Debug::text('Setting OTP IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if (is_array($ids) ) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$pgpplf = new PolicyGroupPremiumPolicyListFactory();
				$pgpplf->getByPolicyGroupId( $this->getId() );

				$tmp_ids = array();
				foreach ($pgpplf as $obj) {
					$id = $obj->getPremiumPolicy();
					Debug::text('Policy ID: '. $obj->getPolicyGroup() .' ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			}

			//Insert new mappings.
			$pplf = new PremiumPolicyListFactory();

			foreach ($ids as $id) {
				if ( isset($ids) AND !in_array($id, $tmp_ids) ) {
					$pgppf = new PolicyGroupPremiumPolicyFactory();
					$pgppf->setPolicyGroup( $this->getId() );
					$pgppf->setPremiumPolicy( $id );

					$obj = $pplf->getById( $id )->getCurrent();

					if ($this->Validator->isTrue(		$this->premiumpolicy_validator_label,
														$pgppf->Validator->isValid(),
														$this->premiumpolicy_validator_msg.' ('. $obj->getName() .')' )) {
						$pgppf->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No OTP IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function getRoundIntervalPolicy() {
		$pgriplf = new PolicyGroupRoundIntervalPolicyListFactory();
		$pgriplf->getByPolicyGroupId( $this->getId() );
		foreach ($pgriplf as $obj) {
			$list[] = $obj->getRoundIntervalPolicy();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setRoundIntervalPolicy($ids) {
		Debug::text('Setting RIP IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if (is_array($ids) ) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$pgriplf = new PolicyGroupRoundIntervalPolicyListFactory();
				$pgriplf->getByPolicyGroupId( $this->getId() );

				$tmp_ids = array();
				foreach ($pgriplf as $obj) {
					$id = $obj->getRoundIntervalPolicy();
					Debug::text('Policy ID: '. $obj->getPolicyGroup() .' ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			}

			//Insert new mappings.
			$riplf = new RoundIntervalPolicyListFactory();

			foreach ($ids as $id) {
				if ( isset($ids) AND !in_array($id, $tmp_ids) ) {
					$pgripf = new PolicyGroupRoundIntervalPolicyFactory();
					$pgripf->setPolicyGroup( $this->getId() );
					$pgripf->setRoundIntervalPolicy( $id );

					$obj = $riplf->getById( $id )->getCurrent();

					if ($this->Validator->isTrue(		$this->roundintervalpolicy_validator_label,
														$pgripf->Validator->isValid(),
														$this->roundintervalpolicy_validator_msg.' ('. $obj->getName() .')' )) {
						$pgripf->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No OTP IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function getMealPolicyID() {
		if ( isset($this->data['meal_policy_id']) ) {
			return $this->data['meal_policy_id'];
		}

		return FALSE;
	}
	function setMealPolicyID($id) {
		$id = trim($id);

		if ( $id == '' OR empty($id) ) {
			$id = NULL;
		}

		$mplf = new MealPolicyListFactory();

		if ( $id == NULL
				OR
				$this->Validator->isResultSetWithRows(	$this->mealpolicy_validator_label,
														$mplf->getByID($id),
														$this->mealpolicy_validator_msg
													) ) {

			$this->data['meal_policy_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getExceptionPolicyControlID() {
		if ( isset($this->data['exception_policy_control_id']) ) {
			return $this->data['exception_policy_control_id'];
		}

		return FALSE;
	}
	function setExceptionPolicyControlID($id) {
		$id = trim($id);

		if ( $id == '' OR empty($id) ) {
			$id = NULL;
		}

		$epclf = new ExceptionPolicyControlListFactory();

		if ( $id == NULL
				OR
				$this->Validator->isResultSetWithRows(	$this->exceptionpolicycontrol_validator_label,
														$epclf->getByID($id),
														$this->exceptionpolicycontrol_validator_msg
													) ) {

			$this->data['exception_policy_control_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getHolidayPolicyID() {
		if ( isset($this->data['holiday_policy_id']) ) {
			return $this->data['holiday_policy_id'];
		}

		return FALSE;
	}
	function setHolidayPolicyID($id) {
		$id = trim($id);

		if ( $id == '' OR empty($id) ) {
			$id = NULL;
		}

		$hplf = new HolidayPolicyListFactory();

		if ( $id == NULL
				OR
				$this->Validator->isResultSetWithRows(	$this->holidaypolicy_validator_label,
														$hplf->getByID($id),
														$this->holidaypolicy_validator_msg
													) ) {

			$this->data['holiday_policy_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function Validate() {
		return TRUE;
	}

	function preSave() {
		return TRUE;
	}

	function postSave() {
		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Policy Group', NULL, $this->getTable() );
	}
}
?>
