<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 420 $
 * $Id: ViewScheduleWeek.php 420 2006-12-08 00:13:05Z ipso $
 * $Date: 2006-12-07 16:13:05 -0800 (Thu, 07 Dec 2006) $
 */
require_once('../../includes/global.inc.php');
require_once(Environment::getBasePath() .'includes/Interface.inc.php');
require_once(Environment::getBasePath() .'classes/misc/arr_multisort.class.php');

//Debug::setVerbosity(11);

if ( !$permission->Check('schedule','enabled')
		OR !( $permission->Check('schedule','view') OR $permission->Check('schedule','view_own') OR $permission->Check('schedule','view_child') ) ) {
	$permission->Redirect( FALSE ); //Redirect
}

$title = 'My Schedule';
$smarty->assign('title', $title);
//BreadCrumb::setCrumb($title);
BreadCrumb::setCrumb($title, str_replace('ViewScheduleWeek.php', 'ViewSchedule.php', $_SERVER['REQUEST_URI']) );

/*
 * Get FORM variables
 */
extract	(FormVariables::GetVariables(
										array	(
												'do',
												'page',
												'sort_column',
												'sort_order',
												'filter_start_date',
												'filter_show_days',
												'filter_user_id'
												) ) );

URLBuilder::setURL($_SERVER['SCRIPT_NAME'],
											array(
													'sort_column' => $sort_column,
													'sort_order' => $sort_order,
													'page' => $page
												) );

if ( $filter_show_days == '' ) {
	$filter_show_days = 1;
}
$filter_show_days = $filter_show_days * 7;

$do = strtolower($do);
switch ($do) {
	case 'view schedule':
		if ( $filter_start_date != '' AND $filter_show_days != '' ) {
			$start_date = $filter_start_date = TTDate::getBeginDayEpoch( TTDate::parseDateTime($filter_start_date) );
			$end_date = $start_date + ($filter_show_days*86400-3601);
		}
	default:
		if ( $permission->Check('schedule','view') ) {
			if ( $filter_user_id == '' ) {
				$filter_user_id = array($current_user->getId());
			}
		} else {
			$filter_user_id = array($current_user->getId());
		}

		if ( $filter_start_date == '' OR $filter_show_days == '' ) {
			$start_date = $filter_start_date = TTDate::getBeginWeekEpoch( TTDate::getTime(), $current_user_prefs->getStartWeekDay() );
			$end_date = $start_date + ($filter_show_days*(86400-3601));
		}

		Debug::text(' Start Date: '. TTDate::getDate('DATE+TIME', $start_date) .' End Date: '. TTDate::getDate('DATE+TIME', $end_date) , __FILE__, __LINE__, __METHOD__,10);

		foreach( $filter_user_id as $user_id ) {
			$sf = new ScheduleFactory();
			$raw_schedule_shifts = $sf->getScheduleArray( $user_id, $start_date, $end_date);
			//var_dump($raw_schedule_shifts);
			if ( is_array($raw_schedule_shifts) ) {
				foreach( $raw_schedule_shifts as $day_epoch => $day_schedule_shifts ) {
					foreach ( $day_schedule_shifts as $day_schedule_shift ) {
						$tmp_schedule_shifts[$day_epoch][$day_schedule_shift['branch']][$day_schedule_shift['department']][] = $day_schedule_shift;
						//$schedule_shifts[$day_epoch][] = $day_schedule_shift;
						if ( $day_schedule_shift['status_id'] == 10 ) { //Working
							if ( isset($schedule_shift_totals[$day_epoch]['total_time']) ) {
								$schedule_shift_totals[$day_epoch]['total_time'] += $day_schedule_shift['total_time'];
							} else {
								$schedule_shift_totals[$day_epoch]['total_time'] = $day_schedule_shift['total_time'];
							}
							$schedule_shift_totals[$day_epoch]['users'][] = $day_schedule_shift['user_id'];

						} elseif ( $day_schedule_shift['status_id'] == 20 ) { //Absent
							if ( isset($schedule_shift_totals[$day_epoch]['absent_total_time']) ) {
								$schedule_shift_totals[$day_epoch]['absent_total_time'] += $day_schedule_shift['total_time'];
							} else {
								$schedule_shift_totals[$day_epoch]['absent_total_time'] = $day_schedule_shift['total_time'];
							}
							$schedule_shift_totals[$day_epoch]['absent_users'][] = $day_schedule_shift['user_id'];
						}
					}
				}
			}
		}

		//Total up employees/time per day.
		if ( isset($schedule_shift_totals) AND isset($total_arr['users']) ) {
			foreach( $schedule_shift_totals as $day_epoch => $total_arr) {
				$schedule_shift_totals[$day_epoch]['total_users'] = count(array_unique($total_arr['users']));
			}
		}
		//var_dump($tmp_schedule_shifts);

		if ( isset($tmp_schedule_shifts) ) {
			//Sort Branches/Departments first
			foreach ( $tmp_schedule_shifts as $day_epoch => $day_tmp_schedule_shift ) {
				ksort($day_tmp_schedule_shift);
				$tmp_schedule_shifts[$day_epoch] = $day_tmp_schedule_shift;

				foreach ( $day_tmp_schedule_shift as $branch => $department_schedule_shifts ) {
					ksort($tmp_schedule_shifts[$day_epoch][$branch]);
				}
			}

			//Sort each department by start time.
			foreach ( $tmp_schedule_shifts as $day_epoch => $day_tmp_schedule_shift ) {
				foreach ( $day_tmp_schedule_shift as $branch => $department_schedule_shifts ) {
					foreach ( $department_schedule_shifts as $department => $department_schedule_shift ) {
						//var_dump($department_schedule_shift);
						$sort = new arr_multisort();
						$sort->setArray($department_schedule_shift);
						$sort->addColumn('start_time', 1);
						$department_schedule_shift = $sort->Sort();
						unset($sort);

						$schedule_shifts[$day_epoch][$branch][$department] = $department_schedule_shift;
					}
				}
			}
		}
		//var_dump($schedule_shifts);

		$calendar_array = TTDate::getCalendarArray($start_date, $end_date, $current_user_prefs->getStartWeekDay(), FALSE);
		//var_dump($calendar_array);
		$smarty->assign_by_ref('calendar_array', $calendar_array);

		$hlf = new HolidayListFactory();
		$holiday_array = $hlf->getArrayByPolicyGroupUserId( $filter_user_id, $start_date, $end_date );
		//var_dump($holiday_array);

		$smarty->assign_by_ref('holidays', $holiday_array);

		$smarty->assign_by_ref('filter_start_date', $filter_start_date);
		$smarty->assign_by_ref('filter_show_days', $filter_show_days);
/*
		$user_options = UserListFactory::getByCompanyIdArray( $current_company->getId(), FALSE, TRUE );
		$smarty->assign_by_ref('user_options', $user_options);
		$show_days_options = array(7 => 7,14 => 14,21 => 21);
		$smarty->assign_by_ref('show_days_options', $show_days_options );
		$smarty->assign_by_ref('filter_user_id', $filter_user_id);
*/
		$smarty->assign_by_ref('schedule_shifts', $schedule_shifts);
		$smarty->assign_by_ref('schedule_shift_totals', $schedule_shift_totals);

		$smarty->assign_by_ref('do', $do );

		break;
}
$smarty->display('schedule/ViewScheduleWeek.tpl');
?>